# -*- coding: utf-8 -*-
# Balazar safari photo
# Copyright (C) 2010-2014 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, struct, math, random, copy, os
import cerealizer
import PIL.Image

import soya3 as soya
import balazar_safari_photo, balazar_safari_photo.globdef as globdef
import balazar_safari_photo.tofu.tofu4soya as tofu
from balazar_safari_photo.flore      import *
from balazar_safari_photo.faune      import *
from balazar_safari_photo.bonus      import *
from balazar_safari_photo.trap       import *
from balazar_safari_photo.level      import *
from balazar_safari_photo.tofu.sides import *

set_side("server")

soya.init()

balazar_safari_photo.flore.load_models()

def load_terrain(parent, map_name):
  terrain = soya.Terrain(parent)
  terrain.from_image(soya.Image.get(map_name))
  nb = 0
  for i in range(terrain.width):
    for j in range(terrain.depth):
      if terrain.get_height(i, j) <= 0.25:
        terrain.set_vertex_option(i, j, 1, 1, 0)
        nb += 1

  print(nb, terrain.width * terrain.depth)
  return terrain

DOWN = soya.Vector(None, 0.0, -1.0, 0.0)
def place(static_world, mobile_world, place_map_filename, Class, min_delta_y = 0.0, max_delta_y = 0.0, color = 0):
  image = PIL.Image.open(os.path.join(globdef.APPDIR, "images", place_map_filename))
  objs = []
  for i in range(image.size[0]):
    for j in range(image.size[1]):
      pixel = image.getpixel((i, j))
      if pixel[color] != 0:
        x = i
        y = 0.0
        z = j
        collision = static_world.raypick(soya.Point(static_world, x, 1000.0, z), DOWN)
        if collision:
          y = (collision[0] % static_world).y
        else:
          print("pixel", i, j, "  ", "x z", x, z)
          raise ValueError
        delta_y = min_delta_y + (pixel[color] / 255.0 * (max_delta_y - min_delta_y))
        if isinstance(Class, soya.Model):
          o = soya.Body(None, Class)
        else:
          o = Class(delta_y)
        if isinstance(o, tofu.Mobile):
          mobile_world.add(o)
        else:
          static_world.add(o)
        o.set_xyz(x, y + delta_y, z)
        objs.append(o)
        print("place", o, o.x, o.y, o.z)
  return objs

def build_materials():
  for f in ["eau_bleu", "x_lumiere_1", "sun", "star", "fx_photo"]:
    print("Build material %s..." % f)
    m = soya.Material.get(f)
    m.additive_blending = 1
    m.save()


def build_champi1():
  name = "champi1"
  static_world = soya.World()
  static_world.filename = "level_static_%s" % name
  mobile_world = soya.World()
  mobile_world.filename = "level_mobile_%s" % name
  
  static_world.weather_data = (8.0, "sun")
  
  terrain = load_terrain(static_world, "map_%s.png" % name)
  terrain.y = -35.0
  terrain.multiply_height(50.0)
  terrain.scale_factor = 1.0
  terrain.texture_factor = 0.3
  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes4" ), 0.0, 50.0,  0.0, 15.0)
  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes"), 0.0, 50.0, 15.0, 40.0)
  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes3"), 0.0, 50.0, 40.0, 180.0)
  terrain.set_material_layer      (soya.Material.get("grass"), 35.0,  50.0)
  terrain.set_material_layer      (soya.Material.get("fond_%s" % name), 0.0,  15.0)
  
  def Champ(delta_y):
    o = soya.Body(None, soya.Model.get("champignon"))
    o.rotate_lateral(len(static_world.children) * 15.0)
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = -4.0, max_delta_y = 6.0)
  
  def Champ(delta_y):
    o = soya.Body(None, soya.Model.get("champignon_double"))
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = 10.0, max_delta_y = 10.0, color = 2)
  
  
  def Champ(delta_y):
    o = FallingMushroom(max_fall = -11.0 - delta_y)
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = -4.0, max_delta_y = 6.0, color = 1)
  
  def Champ(delta_y):
    o = TranpolinMushroom()
    return o
  place(static_world, mobile_world, "map_%s_champ2.png" % name, Champ, min_delta_y = -2.0, max_delta_y = -2.0, color = 2)
  
  def Champ(delta_y):
    o = soya.Body(None, soya.Model.get("arbre_ponpon_tronc"))
    return o
  troncs = place(static_world, mobile_world, "map_%s_champ2.png" % name, Champ, min_delta_y = -0.5, max_delta_y = -0.5, color = 0)
  
  for tronc in troncs:
    o = soya.Body(None, soya.Model.get("arbre_ponpon_feuille"))
    static_world.add(o)
    o.move(tronc)
    o.look_at(soya.Vector(tronc, 0.0, 0.0, -1.0))
  b = GrowingMushroom(soya.Point(None, 33.8249588013, -5.0, 36.7240791321), 0.0025);   mobile_world.add(b)

  b = GrowingMushroom(soya.Point(None, 60.5, -3.0, 43.1), 0.005 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 60.0, -3.0, 43.0), 0.004 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 61.0, -3.0, 42.0), 0.002 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 62.0, -3.0, 44.2), 0.0027);   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 59.6, -3.0, 43.8), 0.001 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 58.5, -3.0, 44.8), 0.003 );   mobile_world.add(b)

  b = GrowingMushroom(soya.Point(None, 59.7176055908, -5.13354492188, 24.9016208649), 0.001 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 87.0157089233, -5.87249565125, 27.19972229  ), 0.0025);   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 110.438873291, -5.52503395081, 28.4925575256), 0.002 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 110.438873291, -5.72604751587, 34.9925613403), 0.003 );   mobile_world.add(b)
  
  b = SmallChest(1); b.set_xyz(13.0, -3.1, 101.34); mobile_world.add(b)
  b = SmallChest(2); b.set_xyz(26.0497169495,  5.0, 9.59568214417); mobile_world.add(b)
  b = SmallChest(3); b.set_xyz(22.9631538391, 28.3, 43.2309074402); mobile_world.add(b)
  b = SmallChest(4); b.set_xyz(59.2698669434, -3.5, 43.7081413269); mobile_world.add(b)
  
  b = Bridge(180.0, "champi2", 0); b.set_xyz(127.338829041, -4.8, 21.5736904144); mobile_world.add(b)
  #b = Door(); b.look_at(soya.Vector(None, 1.0, 0.0, 0.0)); b.set_xyz(137.338829041, -4.7, 21.5736904144); mobile_world.add(b);
  #b = Door(90.0); b.rotate_lateral(90.0); b.set_xyz(137.338829041, -4.7, 21.5736904144); mobile_world.add(b);
  b = Door(90.0, nb_keys = 3); b.set_xyz(137.338829041, -4.7, 21.5736904144); mobile_world.add(b);
  
  b = Champignassien(); b.set_xyz(50.0, 10.0, 50.0); mobile_world.add(b)
  
  b = Monopode(); b.set_xyz(88.8691101074, -14.0, 90.3808898926); mobile_world.add(b)
  b = Monopode(); b.set_xyz(106.832954407, -5.0, 23.3272399902); mobile_world.add(b)
  b = Monopode(); b.set_xyz(39.6990623474,  -7.9, 22.8598766327); mobile_world.add(b)
  
  b = Egg(); b.set_xyz(38.23527145385742, -2.4, 69.09312438964844); mobile_world.add(b)
  
  b = Instruction(3.0, "__tuto_1__", 0); b.set_xyz(23.0, -13.5, 108.0); mobile_world.add(b);
  b = Instruction(3.0, "__tuto_2__", 1); b.set_xyz(10.8, -14.4, 110.5); mobile_world.add(b);
  b = Instruction(3.0, "__tuto_3__", 2); b.set_xyz(53.6, -11.6, 108.0); mobile_world.add(b);
  

  static_world.remove(terrain); static_world.add(terrain) # Put terrain at the end => it will be rendered last, which is faster because it is usually behind other elements
  
  mer = soya.Body(static_world, soya.Model.get("mer3"))
  mer.solid = 0
  mer.y = -19.0
  mer.add_mini_shader(soya.MiniShader.get("water")())
  
  fond = soya.Body(static_world, soya.Model.get("mer_fond2"))
  fond.y = -23.0
  
    

  static_world.save()
  mobile_world.save()
  
  
def build_champi2():
  name = "champi2"
  static_world = soya.World()
  static_world.filename = "level_static_%s" % name
  mobile_world = soya.World()
  mobile_world.filename = "level_mobile_%s" % name
  
  static_world.weather_data = (16.0, "sun", soya.Vector(None, 1.5, -2.0, 2.0))
  
  terrain = load_terrain(static_world, "map_%s.png" % name)
  terrain.y = -40.0 - 11.0
  terrain.multiply_height(50.0)
  terrain.scale_factor = 1.0
  terrain.texture_factor = 0.3
#  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes6" ), 0.0, 50.0,  0.0, 15.0)
  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes" ), 0.0, 80.0,  0.0, 15.0)
#  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes5"), 0.0, 50.0, 15.0, 50.0)
  terrain.set_material_layer_angle(soya.Material.get("feuillesmortes3"), 0.0, 80.0, 15.0, 50.0)
  terrain.set_material_layer_angle(soya.Material.get("rock"), 0.0, 80.0, 50.0, 180.0)
  terrain.set_material_layer      (soya.Material.get("fond_champi2"), 0.0,  15.0)
  
  def Champ(delta_y):
    o = soya.Body(None, soya.Model.get("champignon"))
    o.rotate_lateral(len(static_world.children) * 15.0)
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = -4.0, max_delta_y = 6.0)
  
  def Champ(delta_y):
    o = FallingMushroom(max_fall = -11.0 - delta_y)
    o.rotate_lateral(len(static_world.children) * 15.0)
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = -4.0, max_delta_y = 6.0, color = 1)
  
  def Champ(delta_y):
    o = TranpolinMushroom()
    o.rotate_lateral(len(static_world.children) * 15.0)
    return o
  place(static_world, mobile_world, "map_%s_champ1.png" % name, Champ, min_delta_y = -2.5, max_delta_y = -2.5, color = 2)
   
  def Champ(delta_y):
    #o = soya.Body(None, soya.Model.get("bolet"))
    o = Bolet()
    o.rotate_lateral(len(static_world.children) * 15.0)
    return o
  place(static_world, mobile_world, "map_%s_champ2.png" % name, Champ, min_delta_y = 0.0, max_delta_y = 0.0)
  
  def Champ(delta_y):
    o = soya.Body(None, soya.Model.get("ferme"))
    o.rotate_lateral(90.0)
    return o
  place(static_world, mobile_world, "map_%s_champ2.png" % name, Champ, min_delta_y = -0.8, max_delta_y = -0.8, color = 2)
 
  o = soya.Body(static_world, soya.Model.get("champignon3"))
  o.set_xyz(102.845970154, -0.0, 43.6617889404)

  o = soya.Body(static_world, soya.Model.get("champignon3"))
  o.set_xyz(102.845970154, 7.0, 43.6617889404)

  o = soya.Body(static_world, soya.Model.get("champignon3"))
  o.set_xyz(111.9240448 + 0.2, 3.5, 42.9483604431)

  o = soya.Body(static_world, soya.Model.get("altar"))
  o.set_xyz(83.2732925415, 14.0947971344, 38.8811569214)
  o.rotate_lateral(-30.0)
  
  b = GrowingMushroom(soya.Point(None, 25.7992897034, -10.0294132233, 64.1975784302), 0.001 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 25.7992897034, -10.0294132233, 62.1975784302), 0.1 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 25.7992897034, -10.0294132233, 66.1975784302), 0.05 );   mobile_world.add(b)
  b = GrowingMushroom(soya.Point(None, 31.2992630005, -11.8560295105, 60.9202651978), 0.09 );   mobile_world.add(b)
  
  b = GrowingMushroom(soya.Point(None, 21.2473297119, -12.5187911987, 76.8911437988), 0.05 );   mobile_world.add(b)
  
  b = Bridge(0.0, "champi1", 0)
  b.set_xyz(10.338829041, -5.0, 21.5736904144)
  mobile_world.add(b)
  
  
  static_world.remove(terrain); static_world.add(terrain) # Put terrain at the end => it will be rendered last, which is faster because it is usually behind other elements
  
  
  mer = soya.Body(static_world, soya.Model.get("mer3"))
  mer.solid = 0
  mer.y = -18.0
  mer.add_mini_shader(soya.MiniShader.get("water")())
  
  fond = soya.Body(static_world, soya.Model.get("mer_fond3"))
  fond.y = -39.0
  
  b = SmallChest(1); b.set_xyz(108.09627533, 5.0, 100.331359863); mobile_world.add(b)
  b = SmallChest(2, key_speed = [0.0, 0.2, 0.0]); b.set_xyz(123.09075164794922, -13.395698547363281, 6.061749458312988); mobile_world.add(b)
  b = SmallChest(3, key_speed = [0.0, 0.2, 0.25]); b.set_xyz(85.0157852173, -3.20289993286, 46.6988754272); mobile_world.add(b)
  b = SmallChest(4, key_speed = [0.35, 0.2, 0.05]); b.set_xyz(15.2973327637, 11.210111618, 106.622154236); mobile_world.add(b)
  b = SmallChest(5, key_speed = [0.0, 0.25, 0.2]); b.set_xyz(44.500885009765625, 12.0, 16.0); mobile_world.add(b)
  
  b = Caraconque(); b.set_xyz(-10.0, -10.4836788177, 79.4590682983); mobile_world.add(b); b.current_rotation = 90.0
  b = Echassien(); b.set_xyz(22.778427124, -8.29094696045, 39.5784378052); mobile_world.add(b); b.current_rotation = 90.0
  b = Echassien(); b.set_xyz(104.621955872, 3.87187767029, 103.508209229); mobile_world.add(b)
  b = EchassienPriest(); b.set_xyz(83.2164611816, 23.1427993774, 38.5031585693); mobile_world.add(b);b.current_rotation = 180.0
  
  static_world.save()
  mobile_world.save()

build_materials()
build_champi1()
build_champi2()

