# -*- coding: utf-8 -*-
# Balazar safari photo
# Copyright (C) 2010-2014 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, struct, math, random, copy, os
import cerealizer

import soya3 as soya, soya3.gui.widgets, soya3.gui as gui, soya3.sdlconst as sdlconst
opengl = soya.opengl

import balazar_safari_photo, balazar_safari_photo.globdef as globdef


def close_bubble():
  global BUBBLE
  if BUBBLE:
    try:
      soya.root_widget.remove(BUBBLE)
      return 1
    except: pass
    BUBBLE = None
  return 0
  
BUBBLE = None
class Bubble(gui.Label):
  def __init__(self, text = None, position = None, color = soya.BLACK):
    global BUBBLE
    close_bubble()
    BUBBLE = self
    
    gui.Label.__init__(self, soya.root_widget, text, color = color)
    self.position     = position
    self.extra_height = 0
    self.resize_style = [("percent top", 0.0  ), ("percent left", 0.1  ), ('percent width', 0.8 )]
    self.orig_text = text
    
  def allocate(self, x, y, width, height):
    super().allocate(self.parent.width * 0.1, 0.0, self.parent.width * 0.8, self.height)
    
    drop_it, self.height, self.text = self.font.wordwrap(self.orig_text, self.width - 20)
    self.height += 12.0
    
  def render(self):
    opengl.glPushMatrix()
    opengl.glTranslatef(self.x, self.y, 0.0)
    
    soya.DEFAULT_MATERIAL.activate()
    opengl.glEnable(opengl.GL_BLEND)
    opengl.glColor4f(1.0, 1.0, 1.0, 0.8)
    opengl.glBegin(opengl.GL_POLYGON)
    
    opengl.glVertex3f(-10            ,  0)
      
    opengl.glVertex3f(-10            , self.height - 9)
    opengl.glVertex3f( -7            , self.height - 3)
    opengl.glVertex3f( -1            , self.height)
    
    opengl.glVertex3f(self.width +  1, self.height)
    opengl.glVertex3f(self.width +  7, self.height - 3)
    opengl.glVertex3f(self.width + 10, self.height - 9)
    
    opengl.glVertex3f(self.width + 10,  0)
    
    opengl.glEnd()
    
    if self.position:
      x, y = soya.MAIN_LOOP.camera.coord3d_to_2d(self.position)
      x -= self.x
      y -= self.y
      opengl.glBegin(opengl.GL_TRIANGLES)
      if x < 0.6 * self.width:
        opengl.glVertex3f(self.width * 0.10, self.height)
        opengl.glVertex3f(self.width * 0.05, self.height)
      else:
        opengl.glVertex3f(self.width * 0.95, self.height)
        opengl.glVertex3f(self.width * 0.90, self.height)
      opengl.glColor4f(1.0, 1.0, 1.0, 0.1)
      opengl.glVertex3f(x, y)
      opengl.glEnd()
      
    opengl.glDisable(opengl.GL_BLEND)
    opengl.glPopMatrix()
    
    opengl.glTranslatef(0.0, 5.0, 0.0)
    gui.Label.render(self)
    opengl.glTranslatef(0.0, -5.0, 0.0)
    

class LifeBar(gui.Widget):
  def __init__(self):
    gui.Widget.__init__(self, soya.root_widget)
    
    self.material = soya.Material.get("life")
    self.visible  = 1
    self.life     = 1.0
    
  def allocate(self, x, y, width, height):
    gui.Widget.allocate(self, 5, 5, 37, 261)
    
  def render(self):
    if self.visible:
      
      soya.DEFAULT_MATERIAL.activate()
      opengl.glEnable(opengl.GL_BLEND)
      
      opengl.glBegin(opengl.GL_QUADS)
      opengl.glColor4f(0.9, 0.6, 0.4, 0.7)

      top = int(self.y + 41.0 + (1.0 - self.life) * (self.height - 58.0))
      opengl.glVertex3f(self.x              + 4, self.y + self.height - 18)
      opengl.glVertex3f(self.x + self.width - 4, self.y + self.height - 18)
      opengl.glVertex3f(self.x + self.width - 4, top)
      opengl.glVertex3f(self.x              + 4, top)
      opengl.glEnd()
      
      self.material.activate()
      opengl.glBegin(opengl.GL_QUADS)
      opengl.glColor4f(1.0, 1.0, 1.0, 1.0)
      
      opengl.glTexCoord2f(0.0, 1.0); opengl.glVertex3f(self.x             , self.y + self.height)
      opengl.glTexCoord2f(1.0, 1.0); opengl.glVertex3f(self.x + self.width, self.y + self.height)
      opengl.glTexCoord2f(1.0, 0.0); opengl.glVertex3f(self.x + self.width, self.y)
      opengl.glTexCoord2f(0.0, 0.0); opengl.glVertex3f(self.x             , self.y)
      opengl.glEnd()
      opengl.glDisable(opengl.GL_BLEND)
      
  def set_life(self, life): self.life = life
    
    
    
INVENTORY_FONT = None
class Inventory(gui.Layer):
  def __init__(self, character):
    gui.Layer.__init__(self, None)
    
    self.character = character
    
    f = soya.get_screen_width() / 1024.0
    
    global INVENTORY_FONT
    if not INVENTORY_FONT:
      INVENTORY_FONT = soya.Font(os.path.join(soya.DATADIR, "FreeSans.ttf"), int(16.0 * f), int(18.0 * f))
    #self.root       = soya.set_root_widget(widget.Group())
    #self.background = gui.Image(self.root, soya.Material.get("titre"))
    
    self.scene      = soya.World()
    self.main_loop  = soya.MainLoop(self.scene)
    #self.root       = gui.RootLayer(None); soya.set_root_widget(self.root)
    self.background = gui.Image(self, soya.Material.get("carte"))
    
    self.table           = gui.Table(self, 1, 7)
    self.continue_button = gui.Button(self.table, _(u"Continue"), lambda: soya.MAIN_LOOP.stop(["c"]))
    self.quit_button     = gui.Button(self.table, _(u"Exit")  , lambda: soya.MAIN_LOOP.stop(["q"]))
    self.reset_button    = gui.Button(self.table, _(u"Reset level"), lambda: soya.MAIN_LOOP.stop(["r"]))
    
    self.powers = []
    if "plume" in self.character.powers:
      self.powers.append(gui.Image(self, soya.Material.get("plume2D")))
      
    self.continue_button.set_focus(True)
    self.balazar_pos     = gui.Image(self, soya.Material.get("balazar_mini"))
    
    self.keys = {}
    for level_name in ["champi1", "champi2"]:
      nb_keys = self.character.keys.get(level_name, 0)
      if nb_keys:
        self.keys[level_name] = gui.Image(self, soya.Material.get("clef2D"))
        self.keys[level_name].tex_x2  = float(nb_keys)
        self.keys[level_name].nb_keys = nb_keys
      else:
        self.keys[level_name] = None
        
    #self.quest_photos = {
    #  1 : PhotoImage(self, 112 * f, 358 * f,  7.0, _(u"Monopode")),
    #  2 : PhotoImage(self, 128 * f, 484 * f, -7.0, _(u"Champignassien")),
    #  3 : PhotoImage(self, 294 * f, 486 * f,  4.0, _(u"Prêtre\nÉchassien")),
    #  }
    self.quest_photos = {}
    #for quest_name in character.quests:
    #  quest = balazar_safari_photo.base.QUESTS[quest_name]
    #  self.quest_photos[quest_name] = PhotoImage(self, quest.photo_x * f, quest.photo_y * f, quest.photo_angle, quest)
    for quest in balazar_safari_photo.base.QUESTS.values():
      #if quest.name in character.quests:
        self.quest_photos[quest.name] = PhotoImage(self, quest.photo_x * f, quest.photo_y * f, quest.photo_angle, quest)
      #else:
      #  self.quest_photos[quest.name] = PhotoImage(self, quest.photo_x * f, quest.photo_y * f, quest.photo_angle)
      
    self.photo_images = list(self.quest_photos.values())
    y = 5
    row = 0
    for photo in character.photos:
      if photo.quest_name:
        self.quest_photos[photo.quest_name].set_photo(photo)
      else:
        photo_image = PhotoImage(self, 890 * f, y * f, random.uniform(-10.0, 10.0))
        photo_image.set_photo(photo)
        self.photo_images.append(photo_image)
        y += 85
        
    self.need_render = 1
    self.current_photo_image = None
    
  def set_current_photo_image(self, photo_image):
    if self.current_photo_image: self.current_photo_image.unset_current()
    self.current_photo_image = photo_image
    self.need_render = 1
    
  def show(self):
    soya.set_root_widget(self)
    choice = self.main_loop.main_loop()
    return choice
  
  def allocate(self, x, y, width, height):
    width0  = width
    height0 = height
    
    width  = min(width , height * 1024.0 /  600.0)
    height = min(height, width  *  600.0 / 1024.0)
    gui.Layer.allocate(self, x, y, width, height)
    
    self.background.allocate(0, 0, width0, height0)
    self.background.tex_y2 = float(height0) / width0
    
    f = soya.get_screen_width() / 1024.0
    self.table.calc_size()
    #self.table.allocate(5.0 * f, 5.0 * f, 100.0 * f, height - 10.0 * f)
    #self.table.allocate(5.0 * f, 5.0 * f, 100.0 * f, 0.0 * f + self.table.ideal_height)
    self.table.allocate(5.0 * f, 5.0 * f, self.table.ideal_width, self.table.ideal_height)
    
    if   self.character.level_name == "champi1": self.balazar_pos.allocate(310 * f, 320 * f, 64, 64)
    elif self.character.level_name == "champi2": self.balazar_pos.allocate(400 * f, 340 * f, 64, 64)
    
    for level_name in self.keys:
      if self.keys[level_name]:
        if   level_name == "champi1": self.keys[level_name].allocate(295 * f, 400 * f, 20 * self.keys[level_name].nb_keys, 40)
        elif level_name == "champi2": self.keys[level_name].allocate(400 * f, 400 * f, 20 * self.keys[level_name].nb_keys, 40)
    
    y = 200 * f
    for image in self.powers:
      image.allocate(20, y, 64, 64)
      y += 64
      
  def process_event(self, events):
    focus     = gui.widgets.FOCUSED_WIDGET
    highlight = gui.widgets.HIGHLIGHT_WIDGET
    for event in events:
      if event[0] == sdlconst.WINDOW_RESIZED:
        self.allocate(0, 0, event[1], event[2])
        self.need_render = 1
        continue
      
      if self.current_photo_image:
        if   (event[0] == sdlconst.KEYDOWN) or (event[0] == sdlconst.MOUSEBUTTONDOWN):
          self.set_current_photo_image(None)
      else:
        if   event[0] == sdlconst.KEYDOWN:
          if   event[1] == sdlconst.K_ESCAPE: soya.MAIN_LOOP.stop(["c"]); return
          elif event[1] == "q":               soya.MAIN_LOOP.stop(["q"]); return
          else:
            if focus:
              if focus.on_key_pressed(*event[1:]): continue
            self.table.on_key_pressed(*event[1:])
            
        elif event[0] == sdlconst.MOUSEBUTTONDOWN:
          if (event[2] < self.table.x + self.table.width) and (event[3] < self.table.y + self.table.height):
            self.table.on_mouse_pressed(event[1], event[2], event[3])
          else:
            for photo_image in reversed(self.photo_images):
              if photo_image.on_mouse_pressed(event[1], event[2], event[3]): break
              
        elif event[0] == sdlconst.MOUSEBUTTONUP:
          if (event[2] < self.table.x + self.table.width) and (event[3] < self.table.y + self.table.height):
            self.table.on_mouse_released(event[1], event[2], event[3])
          
        elif event[0] == sdlconst.MOUSEMOTION:
          if (event[2] < self.table.x + self.table.width) and (event[3] < self.table.y + self.table.height):
            self.table.on_mouse_move(*event[1:])
          else:
            if highlight:
              highlight.set_highlight(0)
    #gui.RootLayer.process_event(self, events)
    if (not focus is gui.widgets.FOCUSED_WIDGET) or (not highlight is gui.widgets.HIGHLIGHT_WIDGET):
      self.need_render = 1
      
  def render(self):
    if self.need_render > 0:
      gui.Layer.render(self)
      self.need_render -= 0.5 # Some OpenGL driver requires 2 rendering (due to backbuffering ?)
      
  def widget_begin_round(self):
    self.begin_round()
    events = soya.MAIN_LOOP.raw_events
    self.process_event(events)
  def widget_advance_time(self, p): self.advance_time(p)
  def widget_end_round(self): pass
  
  
class PhotoImage(gui.Image):
  def __init__(self, parent = None, x = 0, y = 0, angle = 0.0, quest = None):
    gui.Image.__init__(self, parent)
    self.angle  = angle
    self.width  = 128.0 * soya.get_screen_width () / 1024.0
    self.height = 128.0 * soya.get_screen_height() / 1024.0# * (float(soya.get_screen_height()) / soya.get_screen_width ())
    self.x      = -self.width  / 2.0
    self.y      = -self.height / 2.0
    self.dx     = x + self.width  / 2.0
    self.dy     = y + self.height / 2.0
    self.quest  = quest
    self.wrapped_quest_label = None
    self.photo  = None
    
  def set_photo(self, photo):
    self.photo    = photo
    self.material = photo.get_mini_material()
    
  def set_current(self):
    if self.photo:
      self.parent.set_current_photo_image(self)
      self.material = self.photo.get_material()
      
  def unset_current(self):
    self.material = self.photo.get_mini_material()
      
  def allocate(self, x, y, width, height):
    pass
  
  def on_mouse_pressed(self, button, x, y):
    if (self.dx + self.x < x < self.dx + self.x + self.width) and (self.dy + self.y < y < self.dy + self.y + self.height):
      self.set_current()
      return 1
    
  def render(self):
    if   self is self.parent.current_photo_image:
      saved = self.width, self.height, self.x, self.y, self.dx, self.dy
      self.width  = 900.0 * soya.get_screen_width () / 1024.0
      self.height = 900.0 * soya.get_screen_height() / 1024.0# * (float(soya.get_screen_height()) / soya.get_screen_width ())
      self.x      = -self.width  / 2.0
      self.y      = -self.height / 2.0
      self.dx     = soya.get_screen_width () / 2.0
      self.dy     = soya.get_screen_height() / 2.0
      border = 15.0
      delta = 0.002
    elif self.parent.current_photo_image: return
    else:
      border = 6.0
      delta = 0.01
      
    opengl.glPushMatrix()
    opengl.glTranslatef(self.dx, self.dy, 0.0)
    opengl.glRotatef(self.angle, 0.0, 0.0, -1.0)
    
    soya.DEFAULT_MATERIAL.activate()
    
    if   self.photo:
      opengl.glEnable(opengl.GL_BLEND)
      for i in range(5):
        opengl.glColor4f(1.0, 1.0, 1.0, 0.2 + 0.2 * i)
        opengl.glBegin(opengl.GL_QUADS)
        opengl.glVertex3f(self.x              - border, self.y - border)
        opengl.glVertex3f(self.x              - border, self.y + self.height + border)
        opengl.glVertex3f(self.x + self.width + border, self.y + self.height + border)
        opengl.glVertex3f(self.x + self.width + border, self.y - border)
        opengl.glEnd()
        border -= 0.7
      opengl.glDisable(opengl.GL_BLEND)

      self.material.activate()
      opengl.glEnable(opengl.GL_BLEND)
      
      border = 0.0
      for i in range(4):
        opengl.glColor4f(1.0, 1.0, 1.0, 0.2 + 0.3 * i)
        opengl.glBegin(opengl.GL_QUADS)
        opengl.glTexCoord2f(self.tex_x1 + delta, self.tex_y1 + delta); opengl.glVertex3f(self.x              + border, self.y + border)
        opengl.glTexCoord2f(self.tex_x1 + delta, self.tex_y2 - delta); opengl.glVertex3f(self.x              + border, self.y + self.height - border)
        opengl.glTexCoord2f(self.tex_x2 - delta, self.tex_y2 - delta); opengl.glVertex3f(self.x + self.width - border, self.y + self.height - border)
        opengl.glTexCoord2f(self.tex_x2 - delta, self.tex_y1 + delta); opengl.glVertex3f(self.x + self.width - border, self.y + border)
        opengl.glEnd()
        border += 0.7
        
      soya.DEFAULT_MATERIAL.activate()
      opengl.glDisable(opengl.GL_BLEND)
      
    elif self.quest.level_name == self.parent.character.level_name:
      opengl.glEnable(opengl.GL_BLEND)
      opengl.glColor4f(1.0, 1.0, 1.0, 0.18)
      #opengl.glLineWidth(3.0)
      #opengl.glBegin(opengl.GL_LINE_LOOP)
      opengl.glBegin(opengl.GL_QUADS)
      opengl.glVertex3f(self.x             , self.y)
      opengl.glVertex3f(self.x             , self.y + self.height)
      opengl.glVertex3f(self.x + self.width, self.y + self.height)
      opengl.glVertex3f(self.x + self.width, self.y)
      opengl.glEnd()
      opengl.glDisable(opengl.GL_BLEND)
      opengl.glLineWidth(1.0)
      
      opengl.glColor4f(1.0, 1.0, 1.0, 1.0)
      if self.wrapped_quest_label is None:
        self.wrapped_quest_label = INVENTORY_FONT.wordwrap(self.quest.get_label(), self.width)
      INVENTORY_FONT.draw(self.wrapped_quest_label[2], self.x + (self.width - self.wrapped_quest_label[0]) / 2.0, -self.wrapped_quest_label[1] / 2.0)
      
    else:
      opengl.glEnable(opengl.GL_BLEND)
      opengl.glColor4f(1.0, 1.0, 1.0, 0.18)
      opengl.glBegin(opengl.GL_QUADS)
      opengl.glVertex3f(self.x             , self.y)
      opengl.glVertex3f(self.x             , self.y + self.height)
      opengl.glVertex3f(self.x + self.width, self.y + self.height)
      opengl.glVertex3f(self.x + self.width, self.y)
      opengl.glEnd()
      opengl.glDisable(opengl.GL_BLEND)
      
    opengl.glPopMatrix()
    
    if self is self.parent.current_photo_image:
      self.width, self.height, self.x, self.y, self.dx, self.dy = saved
