# -*- coding: utf-8 -*-
# Balazar safari photo
# Copyright (C) 2010-2014 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, struct, math, random, copy, os
import cerealizer

import soya3 as soya
import balazar_safari_photo, balazar_safari_photo.globdef as globdef
import balazar_safari_photo.tofu.tofu4soya as tofu
from balazar_safari_photo.tofu.sides import *
from balazar_safari_photo.base       import Platform, Balazar, point, vector, MESSAGE_BUBBLE
from balazar_safari_photo.interface  import Bubble, close_bubble



class Level(tofu.Level):
  def __init__(self, filename):
    tofu.Level.__init__(self)
    
    self.set_filename(filename)
    self._init()
    
    self.mobile_world = soya.World.get("level_mobile_%s" % self.filename)
    self.mobile_world.filename = ""
    
    for mobile in self.mobile_world.children[:]:
      self.mobile_world.remove(mobile)
      self.add_mobile(mobile)
      
  @classmethod
  def load(klass, filename, create = 1):
    import balazar_safari_photo.game
    if balazar_safari_photo.game.RESET_LEVEL: return klass(filename)
    
    return super().load(filename, create)
    
  def loaded(self):
    tofu.Level.loaded(self)
    self._init()
    
  def _init(self):
    self.static_world = soya.World.get("level_static_%s" % self.filename)
    self.add(self.static_world)
    #self.set_program_instance(program())
    #self.add_mini_shader(soya.MiniShader.get("per_pixel_cellshading"))
    
  def __getstate__(self):
    return (self._filename, self.uid, self.round, self.active, self.mobiles)
  
  def __setstate__(self, state):
    self._filename, self.uid, self.round, self.active, self.mobiles = state
    self.state_counter = 0
    
  def set_active(self, active):
    if   active and not self.active:
      soya.MAIN_LOOP.world.add(self)
      soya.MAIN_LOOP.weather.set_time_and_weather(*self.static_world.weather_data)
    elif (not active) and self.active:
      soya.MAIN_LOOP.world.remove(self)
    tofu.Level.set_active(self, active)
    
  def begin_round(self):
    tofu.Level.begin_round(self)
    self.static_world.begin_round()
    
  def advance_time(self, proportion):
    tofu.Level.advance_time(self, proportion)
    self.static_world.advance_time(proportion)
    



BRIDGE_DEMI_DIM = 27.5
class Bridge(Platform, tofu.Mobile):
  def __init__(self, rotation = 0.0, next_level_name = "", bridge_id = 0):
    tofu.Mobile.__init__(self)
    self.rotation        = rotation
    self.next_level_name = next_level_name
    self.bridge_id       = bridge_id
    self._init()
    
  def _init(self):
    Platform._init(self)
    self.set_model(soya.Model.get("pont1"))
    self.rotate_y(self.rotation)
    
  def loaded(self):
    tofu.Mobile.loaded(self)
    self._init()
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.x, self.y, self.z, self.rotation, self.next_level_name, self.bridge_id)
  
  def __setstate__(self, state):
    soya.World.__init__(self)
    self.uid, self.player_name, self.local, self.bot, self.level, self.x, self.y, self.z, self.rotation, self.next_level_name, self.bridge_id = state
    self.last_noticeable_round = 0
    self.last_important_round  = 0
    self._init()

  #@side("single", "server"):
  #def character_on(self, character):
  #  Bubble(_(u"Game saved."))
  #  
  #  return Platform.character_on(self, character)
  
  @side("single", "server")
  def begin_round(self):
    tofu.Mobile.begin_round(self)
    for character in self.characters_on:
      if isinstance(character, Balazar) and (character.life > 0.0):
        point.clone(character)
        point.convert_to(self)
        if point.x < -BRIDGE_DEMI_DIM:
          def plan_change_level():
            old_level = character.level
            new_level = Level.get(self.next_level_name)
            other_bridge = None
            for mobile in new_level.mobiles:
              if isinstance(mobile, Bridge) and (mobile.next_level_name == self.level.filename) and (mobile.bridge_id == self.bridge_id):
                other_bridge = mobile
                break

            old_level.remove_mobile(character)
            
            self.character_off(character)
            
            point.__init__(other_bridge, -BRIDGE_DEMI_DIM + 1.0, 16.0, 0.0)
            point.convert_to(new_level)
            character.y += -self.y + other_bridge.y
            character.x = point.x
            character.z = point.z
            
            new_level.add_mobile(character)
            character.set_chests_visibility()
            character.do_action(" ")
            
            if tofu.has_side("single"): tofu.MAIN_LOOP.reset()
            
            if tofu.MAIN_LOOP.camera:
              tofu.MAIN_LOOP.camera.zap()
              
            character.last_ground_pos.move(character)
            player = tofu.Player.get(character.player_name)
            player.save_position()
            player.save()
            
          soya.MAIN_LOOP.next_round_tasks.append(plan_change_level)
  





cerealizer.register(Level , tofu.SavedInAPathHandler(Level))
cerealizer.register(Bridge)



