# -*- coding: utf-8 -*-
# Balazar safari photo
# Copyright (C) 2010-2014 Jean-Baptiste LAMY
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, struct, math, random, copy, os
import cerealizer

import soya3 as soya, soya3.sdlconst as sdlconst

import balazar_safari_photo, balazar_safari_photo.globdef as globdef
import balazar_safari_photo.tofu.tofu4soya as tofu
from balazar_safari_photo.tofu.sides import *
from balazar_safari_photo.base       import *
from balazar_safari_photo.interface  import Bubble, close_bubble

class Trap(tofu.Mobile):
  def __init__(self, rotation = 0.0):
    tofu.Mobile.__init__(self)
    
    self.rotation = rotation
    self._init()

  def loaded(self):
    soya.World.__init__(self)
    tofu.Mobile.loaded(self)
    self._init()
    
  def _init(self):
    self.rotate_y(self.rotation)
    
  def __getstate__(self):
    return (self.uid, self.player_name, self.local, self.bot, self.level, self.x, self.y, self.z, self.rotation)
    
  def __setstate__(self, state):
    soya.World.__init__(self)
    self.uid, self.player_name, self.local, self.bot, self.level, self.x, self.y, self.z, self.rotation = state
    
  def character_on(self, character): return 0
  def character_off(self, character): pass
  

class Instruction(Trap):
  def __init__(self, radius, text, instruction_nb = -1):
    self.radius         = radius
    self.text           = text
    self.instruction_nb = instruction_nb
    Trap.__init__(self)
    
  def _init(self):
    Trap._init(self)
    self.activated = 0
    
  def __getstate__(self):
    return (Trap.__getstate__(self), self.radius, self.text, self.instruction_nb)
    
  def __setstate__(self, state):
    super_state, self.radius, self.text, self.instruction_nb = state
    Trap.__setstate__(self, super_state)
    
  def do_collisions(self):
    for mobile in self.level.mobiles:
      if isinstance(mobile, Balazar):
        if self.distance_to(mobile) <= self.radius:
          if not self.activated:
            if self.instruction_nb != -1:
              if (self.instruction_nb != mobile.instruction_nb) and (self.instruction_nb != mobile.instruction_nb - 1): continue
              mobile.instruction_nb = self.instruction_nb + 1
            self.activated = 1
            mobile.send_message(MESSAGE_BUBBLE + _(self.text).encode("utf_8"))
            
        else:
          self.activated = 0

cerealizer.register(Instruction)


class Door(Trap, Platform, Dynamic):
  def __init__(self, rotation, nb_keys = 4):
    self.nb_keys = nb_keys
    Trap    .__init__(self, rotation)
    Platform.__init__(self)
    
  def _init(self):
    Trap    ._init(self)
    Platform._init(self)
    
    self.animation = 0
    self.model = soya.Model.get("porte")
    if   self.nb_keys == 3:
      self.battant1 = soya.Body(self, soya.Model.get("porte_battant@porte3" ))
      self.battant2 = soya.Body(self, soya.Model.get("porte_battant2@porte3"))
    elif self.nb_keys == 5:
      self.battant1 = soya.Body(self, soya.Model.get("porte_battant@porte5" ))
      self.battant2 = soya.Body(self, soya.Model.get("porte_battant2@porte5"))
    else:
      self.battant1 = soya.Body(self, soya.Model.get("porte_battant" ))
      self.battant2 = soya.Body(self, soya.Model.get("porte_battant2"))
    self.battant1.x = -3.0 * 1.2
    self.battant2.x =  3.0 * 1.2
    
    self.obstacle = soya.Body(self, soya.Model.get("porte_obstacle"))
    self.obstacle.visible = 0
    
  def __getstate__(self):
    return (Trap.__getstate__(self), self.nb_keys)
    
  def __setstate__(self, state):
    super_state, self.nb_keys = state
    Trap.__setstate__(self, super_state)
    
  def character_on(self, character):
    if isinstance(character, Balazar) and (not character in self.characters_on):
      if character.keys.get(self.level.filename, 0) >= self.nb_keys:
        if self.animation == 0:
          self.animation = 1
          self.obstacle.solid = 0
      else:
        Bubble(_(u"Il faut trouver %s clefs pour ouvrir cette porte !") % self.nb_keys, soya.Point(character, 0.0, 2.5, -0.5))
        
    return Platform.character_on(self, character)
    
  def advance_time(self, proportion):
    Trap.advance_time(self, proportion)
    if self.animation and (self.animation < 22):
      self.battant1.rotate_y(-4.0 * proportion)
      self.battant2.rotate_y( 4.0 * proportion)
      
  def begin_round(self):
    Trap.begin_round(self)
    if self.animation and (self.animation < 22):
      self.animation += 1
      
cerealizer.register(Door)
